package np.org.mpp.eqsolver;
import java.util.ArrayList;
import java.io.*;

import np.org.mpp.configparser.ConfigParsers;
import np.org.mpp.parser.CandidateVerb;

import com.sun.corba.se.impl.orb.ORBConfiguratorImpl.ConfigParser;

/**
 * Solves the equations and inequalities like
 * x+y=1
 * y+z=<1
 * z=0
 * where each variable x y and z can only take the value 0 and 1
 * @author Prajwal Rupakheti, 16 june, 2008
 *
 */

public class EqIqSolver {
	/**
	 * Stores the list of equality equations which are to be solved
	 */
	private ArrayList<String> equalityList;
	/**
	 * Stores the list of inequality equations which are to be solved
	 */
	private ArrayList<String>inEqualityList;	
	/**
	 * Stores the list of every possible combinations of solutions which will be 
	 * tested in equality and inequality equations.
	 */
	private ArrayList<String> solutionList;
	/**
	 * Stores the list of exact solution
	 */
	private ArrayList<String> trueSolutionList;
	private PrintStream logger;
	/**
	 * Sores the total count of variables in the system of equations
	 */
	private int varCount=-1;
	
	private ConfigParsers cfp;
	
	public EqIqSolver(){
		this.equalityList=new ArrayList<String>();
		this.inEqualityList=new ArrayList<String>();
		this.solutionList=new ArrayList<String>();
		this.trueSolutionList=new ArrayList<String>();
		//this.cfp=ConfigParsers.getInstance();
		try{
			//this.logger=new PrintStream(new File("eqlog"));
			System.out.println(ConfigParsers.getValue("eqlog")+"*************");
			//ConfigParsers.size();
			//ConfigParsers.displaytest();
			//System.out.println(cfp.getValue("eqlog")+"**********************");
			this.logger=new PrintStream(new File(ConfigParsers.getValue("eqlog")));
		}
		catch(Exception e ){
			System.out.println(e.toString());
		}
		
	}
	/**
	 * Adds equations to the list
	 * @param equation to be solved
	 * @param flag indicates wether the equation is equality or inequlity
	 * equality if true, inequality otherwise
	 * @throws Exception
	 */
	public void addEquation(String equation,boolean flag)throws Exception{
		int len=equation.length();
		if(this.varCount==-1 || (len==this.varCount)){
			if(flag){
				if(!this.equalityList.contains(equation)){
					this.equalityList.add(equation);
					this.logger.println(equation+" is added as equality. Variable length is :"+ len);
				}
				
			}
			else{
				if(!this.inEqualityList.contains(equation)){
					this.inEqualityList.add(equation);
					this.logger.println(equation+" is added as inequality. Variable length is :"+ len);
				}
				
			}
			this.varCount=len;
		}
		else{
			throw new Exception("Bogus Equation, Number of variables is mismatched");
		}
	}
	/**
	 * Creates list containing all the possible solutions
	 *
	 */
	public void solutionListCreator(){
		String temp;
		this.logger.println();
		double total=Math.pow(2., this.varCount);
		for(int i=0;i<total;i++){
			temp=new String();					
			temp=Integer.toBinaryString(i);
			this.logger.println(temp+" is created as a test solution list");
			if(temp.length()<this.varCount){
				String str=this.getZeroString(this.varCount-temp.length());
				this.logger.println(str+ " need to be concanated to "+temp);
				temp=str.concat(temp);
				this.logger.println(temp+" formed after concatnitation");
			}
			this.solutionList.add(temp);
			this.logger.println(temp+" is added to solution list");
		}
	}
	/**
	 * finds the exact solution from list of all possible solutions
	 *
	 */
	public  void solutionFinder(){
		boolean flag=true;
		String eq;
		String sol;
		int val;
		for(int i=0;i<this.solutionList.size();i++){
			//this.logger.println("\n solutionlist loop :"+i+" size is "+this.solutionList.size()  +"\n");
			sol=this.solutionList.get(i);
			flag=true;
			for(int j=0;j<this.equalityList.size()&&flag;j++){
				eq=this.equalityList.get(j);
				try{
					this.logger.print("Equation is: "+eq+"Soluton is : "+sol+"value --->");
					val=this.valueFinder(sol, eq);
					this.logger.println(val);
					if(val!=1){
						this.logger.println(" solution is not satisfied ");
						flag=false;
					}
				}
				catch(Exception e){
					System.out.println(e.toString());
				}
			}
			for(int k=0;k<this.inEqualityList.size()&&flag;k++){
				eq=this.inEqualityList.get(k);
				try{
					this.logger.print("IqEquation is: "+eq+" Soluton is : "+sol+" value --->");
					val=this.valueFinder(sol, eq);
					this.logger.println(val);
					if(val<0 || val>1){
						this.logger.println(" solution is not satisfied ");
						flag=false;
					}
				}
				catch(Exception e){
					System.out.println(e.toString());
				}
			}
			if(flag){
				this.logger.println("Solution is found:"+sol+" and is added to the list");
				this.trueSolutionList.add(sol);
			}
		}
	
	}
	/**
	 * provides the end value of expression replacing variables of equations with specified values
	 * @param str1 equations's expression
	 * @param str2 variables value
	 * @return end value
	 * @throws Exception
	 */
	private int valueFinder(String str1,String str2) throws Exception{
		if(str1.trim().length()==str2.trim().length()){
			int val=0;		
			String temp1="";
			String temp2="";
			for(int i=0;i<str1.length();i++){
				temp1="";
				temp2="";
				temp1+=str1.charAt(i);
				temp2+=str2.charAt(i);
				val+=(Integer.parseInt(temp1)*Integer.parseInt(temp2));
			}
			return val;
		}
		else{
			throw new Exception("Bogus Equation in value Finder, number of variables is mismatched");
		}
	}
	
	private String getZeroString(int count){
		String temp="";
		for(int i=0;i<count;i++){
			temp+='0';
		}
		return temp;
		
	}
	public ArrayList<String> getFinalSolutionList(){
		return this.trueSolutionList;
	}
	public void displaySolution(){
		for(int i=0;i<this.trueSolutionList.size();i++){
			System.out.println(this.trueSolutionList.get(i));
		}
	}
	
	public static void main(String args[]){
		EqIqSolver eis=new EqIqSolver();
		try{
			eis.addEquation("110", true);
			eis.addEquation("011", true);
			eis.solutionListCreator();
			eis.solutionFinder();
			eis.displaySolution();			
		}
		catch(Exception e){
			System.out.println(e.toString());
		}
	}
}
