package np.org.mpp.parser;
import np.org.mpp.wordbreaker.*;
import np.org.mpp.configparser.*;
import np.org.mpp.frameparser.FrameParser;
import np.org.mpp.frameparser.FrameTransformer;
import np.org.mpp.frameparser.KarakaFrame;
import np.org.mpp.frameparser.MPPCustomizedException;
import np.org.mpp.logger.Logger;

import java.io.File;
import java.util.ArrayList;

public class CandidateVerb {
	private String verb;
	private ArrayList<String>vibhaktis;
	private ArrayList<String>karakas;
	private String configFilePath;
	private String seedFile=null;
	private String postPosFile=null;
	private String generalFrameA=null;
	private String generalFrameB=null;
	private String primaryRootFrame=null;
	private String transformerVerb=null;
	private ConfigParsers cfp;	
	private WordBreaker wb=null;
	private int sourceIndexPosition;
	private int karakaFrameSize;
	/**
	 * Root Frame Parser
	 */
	private FrameParser rootFrameParser=null;
	private FrameParser generalFrameParserA=null;
	private FrameParser generalFrameParserB=null;	
	private FrameTransformer ft = null;
	private KarakaFrame kfRoot=null;
	private KarakaFrame kfVerb=null;
	private KarakaFrame finalTransformedFrame=null;
	private static String TYPE_A="a";
	private static String TYPE_B="b";
	
	//XML tag string constants
	private static String SEED_FILE="seedFile";
	private static String POST_POS_FILE="postPosFile";
	private static String ROOT_FRAME_FILE="primaryRootFrame";
	private static String GENERAL_FRAME_A="generalFrameA";
	private static String GENERAL_FRAME_B="generalFrameB";
	private static String TRANSFORMER_VERB="transFormer";
	
		
	public void init (){
		this.seedFile=cfp.getValue(CandidateVerb.SEED_FILE);
		this.postPosFile=cfp.getValue(CandidateVerb.POST_POS_FILE);
		this.generalFrameA=cfp.getValue(GENERAL_FRAME_A);
		this.generalFrameB=cfp.getValue(GENERAL_FRAME_B);
		this.primaryRootFrame=cfp.getValue(ROOT_FRAME_FILE);
		this.transformerVerb=cfp.getValue(TRANSFORMER_VERB);
	}
	public CandidateVerb(String verb){
		this.verb=verb;
		this.vibhaktis=new ArrayList<String>();
		this.karakas=new ArrayList<String>();
	}
	public void setSourceIndex(int index){
		this.sourceIndexPosition=index;
	}
	public int getSourceIndex(){
		return this.sourceIndexPosition;
	}
	public void setWordBreaker(WordBreaker wb){
		this.wb=wb;
	}
	public int getKarakaFrameSize(){
		return this.finalTransformedFrame.getSize();
	}
	public void setConfigFilePath (String configFilePath) {
		this.configFilePath= configFilePath;
		System.out.println(this.configFilePath);
		this.cfp = ConfigParsers.configParserFactory(this.configFilePath);	
		this.init();
		this.wb = new WordBreaker(this.seedFile ,this.postPosFile );
		try {
			this.generalFrameParserA= FrameParser.getFrameParserInstance(new File(this.generalFrameA));
			this.generalFrameParserB= FrameParser.getFrameParserInstance(new File(this.generalFrameB));			
			this.rootFrameParser = FrameParser.getFrameParserInstance(new File(this.primaryRootFrame));
			
		}catch (Exception e){
			System.out.println(e.toString());
			System.exit(1);
		}	
	}
	
	public String getKaraka(int frameIndex)throws Exception{
		if(frameIndex >this.finalTransformedFrame.getSize()){
			throw new Exception(" karaka index out of bound exception");
		}
		return this.finalTransformedFrame.get(frameIndex, "karaka").trim();
	}
	public String[] getVibhaktis(int frameIndex)throws Exception{
		if(frameIndex >this.finalTransformedFrame.getSize()){
			throw new Exception(" vibhaktis index out of bound exception");
		}
		String vibhakti=this.finalTransformedFrame.get(frameIndex, "vibakti").trim();
		return vibhakti.split(",");
	}
	
	public String getVibhaktisColl(int frameIndex) throws Exception{
		if(frameIndex >this.finalTransformedFrame.getSize()){
			throw new Exception(" vibhaktis index out of bound exception");
		}
		String vibhakti=this.finalTransformedFrame.get(frameIndex, "vibakti").trim();
		return vibhakti;
	}
	public String getOpt(int frameIndex)throws Exception{
		if(frameIndex >this.finalTransformedFrame.getSize()){
			throw new Exception(" karaka index out of bound exception");
		}
		return this.finalTransformedFrame.get(frameIndex, "mandatory").trim();
	}
	public String getLexType(int frameIndex)throws Exception{
		if(frameIndex >this.finalTransformedFrame.getSize()){
			throw new Exception(" karaka index out of bound exception");
		}
		return this.finalTransformedFrame.get(frameIndex, "lextype").trim();
	}
	
	public String getVerb(){
		return this.verb;
	}
	
	/**
	 *Add methods and module to extract information form wordbreaker 
	 * @param candidate The verb candidate for processing.
	 * @throws Exception For other group of verbs.
	 */

	public void breakAndLoad(String candidate)throws Exception{
		//word breaker part
		this.wb.load();
		this.wb.setWord(candidate);
		this.wb.breakWord();
		if(this.wb.getSeed()==null || this.wb.getSuffix()==null || this.wb.getGroup()==null){
			throw new Exception("Unable to break verb: "+this.verb+
					" Following info are found: "+
					"seed:"+this.wb.getSeed()+" suffix:"+this.wb.getSuffix()+
					"type: "+this.wb.getGroup());
		}
		String type = wb.getGroup();
		String suffix = this.wb.getSuffix();
		Logger.log("Seed : "+this.wb.getSeed()+" Suffix : "+this.wb.getSuffix()+" Group: "+this.wb.getGroup());
		
		if (type.equals(TYPE_A))	{		
			try{
				this.generalFrameParserA.searchPostPosition(suffix);
				this.generalFrameParserA.loadFrame();	
				this.kfVerb=this.generalFrameParserA.getFrame();
			}
			catch(Exception e){
				Logger.log(e.toString());
			}			
			
		}else if (type.equals(TYPE_B)) {
			Logger.log("entered "+TYPE_A);
			try{
				this.generalFrameParserB.searchPostPosition(suffix);
				this.generalFrameParserB.loadFrame();			
				this.kfVerb=this.generalFrameParserB.getFrame();
			}
			catch(Exception e){
				Logger.log(e.toString());
			}
		}else {
			throw new Exception ("Could not parse Xml, Wrong Type");
		}
		//parse frame
		try {			
			this.rootFrameParser.searchPostPosition(this.transformerVerb);
			this.rootFrameParser.loadFrame();
			this.kfRoot=this.rootFrameParser.getFrame();
			
			//transform
			this.ft=new FrameTransformer(this.kfRoot,this.kfVerb);
			this.ft.transform();
			
			this.finalTransformedFrame=ft.getTransformedFrame();
			
		} catch (MPPCustomizedException mce){
			System.out.println(mce.toString());
		}
		
	}
		
	public static void main ( String args[]) {
		String candidate= "दियो" ;
		System.out.println(candidate);
		CandidateVerb cv = new CandidateVerb(candidate);
		cv.setConfigFilePath(args[0]);
		try {
			cv.breakAndLoad(candidate);
			Logger.log(cv.getKaraka(0));
			Logger.log(cv.getLexType(0));
			String str[]=cv.getVibhaktis(0);
			Logger.log(str.length+" "+str[1]);
		}catch (Exception e) {
			System.out.println(e.toString());
		}
	}
}
