package np.org.mpp.parser;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import np.org.mpp.logger.Logger;

public class ChunkCapsule {
	private String chunkTag;
	private String chunkHeadWOrd;
	private String chunkHeadPos;
	private ArrayList<ArcVariable> incomingVarList;
	private ArrayList<ArcVariable> outGoingVarList;
	
	private HashMap<String, ArrayList<ArcVariable>> optOutGoingHash;
	private HashMap<String, ArrayList<ArcVariable>> mandOutGoingHash;

	private ArrayList <WordPosPair>wordPosPairList;
	ChunkCapsule(String chunkTag){
		this.chunkTag=chunkTag;
		this.wordPosPairList=new ArrayList<WordPosPair>();
		this.incomingVarList=new ArrayList<ArcVariable>();
		this.outGoingVarList=new ArrayList<ArcVariable>();
		
		this.optOutGoingHash=new HashMap<String,ArrayList<ArcVariable>>();
		this.mandOutGoingHash=new HashMap<String, ArrayList<ArcVariable>>();
	}
	ChunkCapsule(){	
		this.wordPosPairList=new ArrayList<WordPosPair>();
	}
	public void setChunkTag(String chunkTag){
		this.chunkTag=chunkTag;
	}
	public void loadWordPosPair(WordPosPair wordPosPair){
		this.wordPosPairList.add(wordPosPair);
	}
	private void findChunkHead(){
		int end=this.wordPosPairList.size()-1;
		if(this.wordPosPairList.get(end).getTag().equals("POP")){
			
			this.chunkHeadPos=this.wordPosPairList.get(end-1).getTag();
			this.chunkHeadWOrd=this.wordPosPairList.get(end-1).getWord();
		}
		else{
			this.chunkHeadPos=this.wordPosPairList.get(end).getTag();
			this.chunkHeadWOrd=this.wordPosPairList.get(end).getWord();
		}
	}
	public String getVibhakti(){
		int end=this.wordPosPairList.size()-1;
		if(this.wordPosPairList.get(end).getTag().equals("POP")){
			return this.wordPosPairList.get(end).getWord();
		}
		else{
			return "PHI";
		}
	}
	public String getChunkHeadWord(){
		this.findChunkHead();
		return this.chunkHeadWOrd;
	}
	public String getChunkHeadPos(){
		return this.chunkHeadPos;
	}
	public String getChunkTag(){
		return this.chunkTag;
	}
	
	public int getListLength(){
		return this.wordPosPairList.size();
	}
	public String getWord(int i){
		return this.wordPosPairList.get(i).getWord();
	}
	public String getTag(int i){
		return this.wordPosPairList.get(i).getTag();
	}
	public void addIncomingVar(ArcVariable av){
		this.incomingVarList.add(av);
		Logger.log("\nIncoming arc Variable added to chunk: "+this.getAnnonatedChunk());
		av.logArc();
		Logger.log("\n");
	}
	public void addOutGoingVar(ArcVariable av){
		this.outGoingVarList.add(av);
		Logger.log("\nOutGoing arc Variable added to chunk: "+this.getAnnonatedChunk());
		av.logArc();
		Logger.log("\n");
	}
	public String getAnnonatedChunk(){
		int i;
		String str="{";
		String temp="";
		WordPosPair wpp;
		for(i=0;i<this.wordPosPairList.size();i++){
			wpp=this.wordPosPairList.get(i);
			temp=wpp.getWord()+"/"+wpp.getTag()+" ";
			str+=temp;
		}
		str+="}";
		return str.concat("/"+this.chunkTag);
	}
	public void populateHash(){
		int i;
		String karaka;
		String opt;
		ArrayList<ArcVariable> tempList;
		ArcVariable av;
		for(i=0;i<this.outGoingVarList.size();i++){
			av=this.outGoingVarList.get(i);
			opt=av.optString();
			karaka=av.getKaraka();
			if(karaka!=null && opt.equals("YES")){
				if(!this.mandOutGoingHash.containsKey(karaka)){
					tempList=new ArrayList<ArcVariable>();
					tempList.add(av);
					this.mandOutGoingHash.put(karaka, tempList);
				}
				else{
					tempList=this.mandOutGoingHash.get(karaka);
					tempList.add(av);					
				}
			}
			else if(karaka!=null && opt.equals("NO")){
				if(!this.optOutGoingHash.containsKey(karaka)){
					tempList=new ArrayList<ArcVariable>();
					tempList.add(av);
					this.optOutGoingHash.put(karaka, tempList);
				}
				else{
					tempList=this.optOutGoingHash.get(karaka);
					tempList.add(av);					
				}
			}
		}
	}
	public String incomingEquation(){
		if(this.incomingVarList.size()>0){
			String equation= this.generateEquation(this.incomingVarList);	
			return equation;
		}
		else{
			return null;
		}
	}
	public String[] outgoingMandatoryEquations(){		
		Set<String> set;
		set=this.mandOutGoingHash.keySet();
		String equations[]=new String[set.size()];
		Iterator<String> itr;
		itr=set.iterator();
		int i=0;
		while(itr.hasNext()){
			String key=itr.next();			
			equations[i]=this.generateEquation(this.mandOutGoingHash.get(key));
			i++;
		}
		return equations;
	}
	
	public String[] outgoingOptionalEquations(){
		Set<String> set;
		set=this.optOutGoingHash.keySet();
		String equations[]=new String[set.size()];
		Iterator<String> itr;
		itr=set.iterator();
		int i=0;
		while(itr.hasNext()){
			String key=itr.next();
			equations[i]=this.generateEquation(this.optOutGoingHash.get(key));
			i++;
		}
		return equations;
	}
	private String generateEquation(ArrayList<ArcVariable> av){
		String equation="";
		int i;		
		int varIndexes[]=new int[av.size()];
		for(i=0;i<av.size();i++){
			int varIndex=av.get(i).getVarIndex();
			varIndexes[i]=varIndex;			
		}
		for(i=0;i<ArcVariable.VARMAX;i++){
			if(this.intSearch(varIndexes, i)){
				equation+="1";
			}
			else{
				equation+="0";
			}
		}
	
		return equation;
	}
	private boolean intSearch(int arr[], int item){
		boolean flag=false;
		for(int i=0;i<arr.length;i++){
			if(item==arr[i]){
				flag=true;
				break;
			}
		}
		return flag;
	}
	
}

