package np.org.mpp.parser;



import java.util.ArrayList;

import np.org.mpp.eqsolver.*;
import np.org.mpp.logger.Logger;

public class ChunkRelationFinder {
	
	private ArrayList<ChunkCapsule> chunkCapsuleList;	
	private ArrayList<CandidateVerb> verbList;
	private ArrayList<ArcVariable> arcVariableList;	
	
	private ArrayList<ArrayList<ArcVariable>> arcVariableFinalParseList;
	private EqIqSolver equationSolver;
	private ArrayList<String> solutionList;
	private boolean parseFlag=false;
	
	public String configFilePath;
	
	private static String LEXTYPE="lextype";
	private static String KARAKA="karaka";
	private static String OPT="mandatory";
	private static String VIBHAKTI="vibhakti";
	
	
	
	public ChunkRelationFinder(ArrayList<ChunkCapsule> chunkCapsuleList){
		this.chunkCapsuleList=chunkCapsuleList;
		this.verbList=new ArrayList<CandidateVerb>();		
		this.arcVariableList=new ArrayList<ArcVariable>();
		this.arcVariableFinalParseList=new ArrayList<ArrayList<ArcVariable>>();
		this.equationSolver=new EqIqSolver();
	}
	
	public void setConfigFilePath(String configFilePath){
		this.configFilePath=configFilePath;
	}
	
	public void start(){
		this.loadCandidateVerb();
		if(this.verbList.size()>0){
			this.findArcs();
			this.solve();		
			this.generateFinalParse();
		}
	}
	private void loadCandidateVerb(){
		int i;		
		for(i=0;i<this.chunkCapsuleList.size();i++){
		
			if(this.chunkCapsuleList.get(i).getChunkTag().equals("VCH")){
				CandidateVerb cv=new CandidateVerb(this.chunkCapsuleList.get(i).getChunkHeadWord());
				Logger.log("Candidate verb found :"+ cv.getVerb());
				cv.setSourceIndex(i);				
				cv.setConfigFilePath(this.configFilePath);
				try {
					cv.breakAndLoad(this.chunkCapsuleList.get(i).getChunkHeadWord());
					this.verbList.add(cv);
				}catch (Exception e) {
					Logger.log(cv.getVerb()+" is not loaded");
					Logger.log(e.toString());
				}
				
			}
		}		
	}
	
	
	private void findArcs(){
		int i;
		int j;
		int k;
		CandidateVerb cv;
		ChunkCapsule cc;
		ChunkCapsule tempCapsule;
		String lexTypeVerb; 
		String opt;
		String vibhaktis[];// of verb candidate
		String karaka;
		
		String vibhakti;// of chunk capsule
		String lexTypeChunk;
		
		int varCount=0;
		ArcVariable arcVar;
		for(i=0;i<this.verbList.size();i++){
			cv=this.verbList.get(i);
			for(j=0;j<this.chunkCapsuleList.size();j++){
				cc=this.chunkCapsuleList.get(j);
				if(j<cv.getSourceIndex() && !cc.getChunkTag().equals("VCH")){
					Logger.log("\nRelationship finding between VERB: "+cv.getVerb()+
							" and CHUNK: "+cc.getAnnonatedChunk()+" starts.**************************");
					vibhakti=cc.getVibhakti();
					lexTypeChunk=cc.getChunkTag().trim();
					
					Logger.log(cc.getAnnonatedChunk()+"--> vibhakti: "+vibhakti
							+" lextype: "+lexTypeChunk);
					for(k=0;k<cv.getKarakaFrameSize();k++){
						try{
							Logger.log("\nStart of Frame "+k+" for Verb--->"+cv.getVerb());
							vibhaktis=cv.getVibhaktis(k); //exception method
							opt=cv.getOpt(k);
							karaka=cv.getKaraka(k);
							lexTypeVerb=cv.getLexType(k).trim();
							Logger.log("Vibhaktis: "+cv.getVibhaktisColl(k));
							Logger.log("karaka: "+cv.getKaraka(k));
							Logger.log("Lextype: "+cv.getLexType(k));
							Logger.log("Demand type: "+cv.getOpt(k));
							if(this.vibhaktiSearch(vibhaktis, vibhakti)&& lexTypeChunk.equals(lexTypeVerb)){
								Logger.log("\nRelation established.");
								arcVar=new ArcVariable(cv.getSourceIndex(),j,varCount,opt,karaka);
								tempCapsule=this.chunkCapsuleList.get(cv.getSourceIndex());
								arcVar.setSourceNode(tempCapsule);
								arcVar.setDestNode(cc);
								cc.addIncomingVar(arcVar);								
								tempCapsule.addOutGoingVar(arcVar);
								this.arcVariableList.add(arcVar);								
								varCount++;
							}
							Logger.log("End of Frame "+k+" for Verb--->"+cv.getVerb());
						}
						catch(Exception e){
							System.out.println(e.toString());
						}
					}
					Logger.log("\nRelationship finding between VERB: "+cv.getVerb()+
							" and CHUNK: "+cc.getAnnonatedChunk()+" ends. **************************");	
				}
			}
		}
		
	}
	
	private void solve(){
		int i;
		int j;		
		
		ChunkCapsule cc;
		String incomingEq;
		String[] outGoingMand;
		String[] outGoingOpt;
		
		for(i=0;i<this.chunkCapsuleList.size();i++){
			cc=this.chunkCapsuleList.get(i);
			cc.populateHash();
			incomingEq=cc.incomingEquation();
			outGoingMand=cc.outgoingMandatoryEquations();
			outGoingOpt=cc.outgoingOptionalEquations();
			try{
				if(incomingEq !=null){
					this.equationSolver.addEquation(incomingEq, true);
					Logger.log("\nIncoming eq: "+incomingEq+" Added");
				}
				
				for(j=0;j<outGoingMand.length;j++){					
					this.equationSolver.addEquation(outGoingMand[j], true);					
					Logger.log("\nOut going mandatory eq: "+outGoingMand[j]+" Added");
				}
				for(j=0;j<outGoingOpt.length;j++){
					this.equationSolver.addEquation(outGoingOpt[j], false);
					Logger.log("\nOut going optional eq: "+outGoingOpt[j]+" Added");
				}
			}
			catch(Exception e){
				Logger.log(e.toString());
			}			
		}
		this.equationSolver.solutionListCreator();
		this.equationSolver.solutionFinder();
		this.solutionList=this.equationSolver.getFinalSolutionList();	
		if(this.solutionList.size()>0){
			Logger.log("\nEquation is solved. Following are the solutoins:");
			for(int m=0;m<this.solutionList.size();m++){
				Logger.log(this.solutionList.get(m));
			}
			
			this.parseFlag=true;
		}
		else{
			this.parseFlag=false;
		}
	}
	
	private void generateFinalParse(){
		int i;
		int j;
		String solution;
		ArrayList<ArcVariable> singleSol;
		for(i=0;i<this.solutionList.size();i++){
			singleSol=new ArrayList<ArcVariable>();
			solution=this.solutionList.get(i);
			for(j=0;j<solution.length();j++){
				if(solution.charAt(j)=='1'){
					singleSol.add(this.arcVariableList.get(j));
				}
			}
			this.arcVariableFinalParseList.add(singleSol);
		}			
	}
	
	public boolean isParsed(){
		return this.parseFlag;
	}
	public void displayParse(){
		if(!this.isParsed()){
			Logger.log(" \n\n Sentence not parsed: ill formed sentence.");
			return;
		}
		int i;
		int j;
		ArrayList<ArcVariable> singleSol;
		ArcVariable tempVar;
		ChunkCapsule sourceCc;
		ChunkCapsule destCc;
		for(i=0;i<this.arcVariableFinalParseList.size();i++){
			singleSol=this.arcVariableFinalParseList.get(i); 
			Logger.log("\n\nSolution parse: "+ i+ " starts *****************");
			for(j=0;j<singleSol.size();j++){
				tempVar=singleSol.get(j);
				sourceCc=tempVar.getSourceNode();
				destCc=tempVar.getDestNode();
				Logger.log(sourceCc.getAnnonatedChunk()+"-->"+tempVar.getKaraka()+
									"<--"+destCc.getAnnonatedChunk());
			}
			Logger.log("Solution parse: "+ i+ " ends *****************");
		}
	}
	
	
	private	boolean vibhaktiSearch(String[] vibhakits, String vibhakti){
		boolean flag=false;
		int i;
		for(i=0;i<vibhakits.length;i++){
			if(vibhakti.trim().equals(vibhakits[i].trim())){
				flag=true;
				break;
			}
		}
		return flag;
	}
	
	
}
