package np.org.mpp.wordbreaker;

/**
 *
 * @author niraj
 */
import java.util.*;
import java.io.*;

//import javax.swing.JOptionPane;

/**
 * The WordBreaker Class This class includes many different functions:
 * WordBreaker(), Load(), breakWord(), setWord(), identifySuffix(), getSuffix(),
 * getSeed(), trunkSuffix().This class mainly identifies a word from the user
 * and reads the lex Suffix file. Every node of the word is checked with the lex
 * Suffix file. If match is found then that matched word is stored in the
 * suffix. Truncating the suffix from the word gives us the root word.
 * 
 * @author
 * @param void
 * @return void
 */
public class WordBreaker {
	/**
	 * Scans the seed in the lex Seed file.
	 */
	private Scanner seedSc;
	/**
	 * Scans the suffix in the lex Suffix file.
	 */
//	private BufferedReader seedReader;
	
	private Scanner suffSc;

	/**
	 * stores the possible list of the suffix
	 */
	private ArrayList<String> suffList;
	/**
	 * stores the possible list of the seeds
	 */
	private ArrayList<String> seedList;
	/**
	 * stores the seed as key and its associated group as value
	 */
	private Hashtable<String, String> seedHash;
	/**
	 * Stores the seed with the suffix truncated and remains unchaned throughout
	 * the program
	 */
	String rootSeed;
	/**
	 * stores the seed of the word
	 */
	private String seed;
	/**
	 * stores the suffix of the word
	 */
	private String suffix = null;
	/**
	 * Stores the input word to be fragmented
	 * 
	 */
	private String word;

	/**
	 * holds the current suffix search index
	 */
	public int suffixSearchIndexCurr;
	/**
	 * Reads characters from Seed Lexicon file and suffix file initializes the
	 * constructor <b>WordBreaker</b>
	 * 
	 * @param seedFile
	 *            : the seed Lexicon file
	 * @param suffFile
	 *            : the suffix file
	 */

	/**
	 * logger
	 */
	PrintStream logger;

	public WordBreaker(String seedFile, String suffFile) {
		try {
			this.seedSc = new Scanner(new File(seedFile), "UTF-8");
//			this.seedReader=new BufferedReader(new FileReader(new File(seedFile)));
			this.suffSc = new Scanner(new File(suffFile), "UTF-8");
			this.logger = new PrintStream(new File("log"), "UTF-8");
		} catch (Exception e) {
			System.out.println(e);
		}
		this.suffList = new ArrayList<String>();
		this.seedList = new ArrayList<String>();
		this.seedHash = new Hashtable<String, String>();
		this.suffixSearchIndexCurr = 0;
	}

	/**
	 * Loads seed lexicon and suffix file into suffList and seedList. As long as
	 * the file has a next line,this function loads the next line in the seed
	 * file into a array list called seedList. Similarly, this function also
	 * loads the next line in the suffix file into a array list called suffList,
	 * as long as there is next line in the lex suffix file.
	 * 
	 * @param void
	 * @return void
	 * @author dipti and srijana
	 */
	public void load() {
		String line;
		String tokens[];
		
		while (this.seedSc.hasNext()) {
			line = seedSc.nextLine().trim();			
			tokens = line.split(" ");
			if (tokens.length > 1) {			
				this.seedHash.put(tokens[0], tokens[1]);
				seedList.add(tokens[0]);
				this.logger.println(tokens[0]);				
			}
		}		
		
		seedSc.close();
		while (suffSc.hasNextLine()) {
			line = suffSc.nextLine();
			suffList.add(line);
		}
		suffSc.close();
	}

	/**
	 * Breaks word into seed and suffix
	 * 
	 * @param void
	 * @return void
	 */
	public void breakWord() {
		boolean flag;
		flag = this.identifySuffix();
		if (flag) {
			this.trunkSuffix();
			flag = this.identifySeed();
		}
	}

	/**
	 * Initializes the word This function sets the identified word from the user
	 * into the variable word.
	 * 
	 * @param word
	 *            : stores the identified word from user.
	 */
	public void setWord(String word) { // sets the given word
		this.word = word;
	}

	/**
	 * identifies the suffix from file
	 * 
	 * @return boolean : true if the suffix is present on the suffix list
	 */
	private boolean identifySuffix() {
		String tempSuff;
		boolean flag = false;
		for (int i = 0; i < suffList.size(); i++) {
			tempSuff = suffList.get(i);
			if (word.endsWith(tempSuff)) {
				logger.println("(Suffix  found: )" + this.word + " " + tempSuff
						+ "***************************************");
				suffix = tempSuff;
				flag = true;
				// new edited
				if (searchSeedLexicon(this.word, tempSuff)) {
					break;
				} else {
					continue;
				}
			} else {
				logger.println("(Suffix not found: )" + this.word + " "
						+ tempSuff);
			}
		}
		return flag;
	}

	/**
	 * Searches Seed for the corresponding suffix.
	 * 
	 * @param str
	 *            The word.
	 * @param suf
	 *            The suffix.
	 * @return
	 */
	private boolean searchSeedLexicon(String str, String suf) {
		/**
		 * newSeed for new seed after deleting the suffix tempSeed for storing
		 * temp suffix read from seedList
		 */
		String newSeed = new String(), tempSeed = null;
		newSeed = str.substring(0, str.length() - suf.length());
		for (int i = 0; i < seedList.size(); i++) {
			tempSeed = seedList.get(i);
			if (newSeed.equals(tempSeed)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Distinguish suffix from seed.
	 * 
	 * @param void
	 * @return void
	 */
	private void trunkSuffix() {
		this.seed = new String();
		char ch[] = new char[this.word.indexOf(this.suffix)];
		for (int i = 0; i < ch.length; i++) {
			ch[i] = this.word.charAt(i);
			this.seed += ch[i];
			logger.println(ch[i]);
		}
		this.rootSeed = new String(this.seed);
		logger.println("word:" + this.word + " Seed: " + seed.trim()
				+ " Suffix :" + this.suffix + "Start index"
				+ this.word.indexOf(this.suffix));
	}

	/**
	 * Identifies the seed.
	 * 
	 * @param void
	 * @return void
	 */
	private boolean identifySeed() {
		boolean flag = true;
		flag = this.searchSeed();
		if (flag) {
			return flag;
		}
		if (!flag) {
			this.seed = this.seed.substring(0, this.seed.length() - 1);
		}
		flag = this.searchSeed();
		if (!flag) {
			this.seed = new String(this.rootSeed);
			this.seed += '्';
			flag = this.searchSeed();
			if (!flag) {
				this.seed = null;
			}
		}
		return flag;

	}

	/**
	 * Checks whether there's seed in the list.
	 * 
	 * @param void
	 * @return boolean: returns true if there's seed in the list
	 */
	private boolean searchSeed() {
		boolean flag = false;
		String tempSeed;
		for (int i = 0; i < seedList.size(); i++) {
			tempSeed = seedList.get(i);
			if (this.seed.equals(tempSeed)) {
				logger.println("Seed List -------------->(matched) "
						+ this.seed + " " + tempSeed);
				flag = true;
				// return flag;
				// tempSeed = seedList.get(i);
				// continue;
				break;
			} else {
				logger.println("Seed List -------------->(not matched) "
						+ this.seed + " " + tempSeed);
			}
		}
		return flag;
	}

	/**
	 * @param void
	 * @return String: returns the suffix
	 */
	public String getSuffix() {
		return this.suffix;
	}

	/**
	 * @param void
	 * @return String: returns the seed
	 */
	public String getSeed() {
		return this.seed;
	}

	public String getGroup() {
		if (this.seed != null) {
			return this.seedHash.get(this.seed);
		} else {
			return null;
		}
	}
}// end WordBreaker
